#!/bin/bash
# vim: noet

set -eu

PREFIX="$(realpath "$(dirname "${BASH_SOURCE[0]}")")"
CONF="$PREFIX/nachos.cfg"

trap error ERR

error() {
	echo "error: failed at ${BASH_SOURCE[0]}:${BASH_LINENO[0]}" >&2
	exit 1
}

if ! systemctl -q is-active mariadb; then
	cat <<-EOF 1>&2

	  ERROR: mariadb.service is not running, cannot initialize database.
	  Start the service and run this script manually: $0

	EOF
	exit 1
fi

if ! mysql -u root -D mysql -e 'quit'; then
	cat <<-EOF 1>&2

	  ERROR: Cannot initialize MySQL database for "nachos" with
	  'root'@'localhost'.  Configure password for the MySQL user above in
	  $HOME/.my.cnf and run this script manually: $0

	EOF
	exit 1
fi

mysql -u root -e "CREATE DATABASE IF NOT EXISTS nachos CHARACTER SET = 'utf8' COLLATE = 'utf8_unicode_ci';"

# There is no "CREATE USER IF NOT EXISTS" in MariaDB 5.5, so check for existence manually.
if ! mysql -B -N -u root -e "SELECT 'exists' FROM mysql.user WHERE User = 'nachos' AND Host = 'localhost'" \
		| grep -q exists; then
	mysql -B -N -u root -e "CREATE USER 'nachos'@'localhost' IDENTIFIED BY 'NACHOS_DBPASS'"
	echo "INFO: Created database user for nachos"
fi
# Make sure nachos user has required privileges.
mysql -u root -e "GRANT ALL PRIVILEGES ON nachos.* TO 'nachos'@'localhost'"
mysql -u root -e "GRANT ALL PRIVILEGES ON nacoma.* TO 'nachos'@'localhost'"

# Remove 'nachos'@'%' created in previous versions if it still exists.
if mysql -B -N -u root -e "SELECT 'exists' FROM mysql.user WHERE User = 'nachos' AND Host = '%'" \
		| grep -q exists; then
	mysql -u root -e "DROP USER 'nachos'@'%'"
fi

# Check if access works with what is configured in nachos.cfg
for db in nachos nacoma; do
	# Extract USERNAME/PASSWORD from the line matching DBNAME from nachos config:
	# connection = mysql+pymysql://USERNAME:PASSWORD@localhost/DBNAME?unix_socket=/var/lib/mysql/mysql.sock
	IFS=$'\t' read -r username password < <(perl -lne 'print "$1\t$2" if m%://(\w+):(.+)@.*'"${db}%" $CONF)
	mysql -u "$username" --password="$password" -e "use $db"
done

if [ -z "${RPM_INSTALL_PREFIX:-}" ]; then
	echo "Successfully initialized nachos database!"
fi
